﻿using Microsoft.VisualBasic;
using System;
using System.Globalization;
using System.IO;
using System.Reflection.PortableExecutable;
using System.Runtime.CompilerServices;
using System.Text;
using System.Xml.Linq;


namespace DmrIdDbConverter
{
    internal class Program
    {
        public static Dictionary<string, string> foreign_characters = new Dictionary<string, string>
        {
            { "æǽ", "ae" },
            { "œ", "oe" },
            { "ÄÀÁÂÃÄÅǺĀĂĄǍΑΆẢẠẦẪẨẬẰẮẴẲẶА", "A" },
            { "äàáâãåǻāăąǎªαάảạầấẫẩậằắẵẳặа", "a" },
            { "Б", "B" },
            { "б", "b" },
            { "ÇĆĈĊČЋ", "C" },
            { "çćĉċčћ", "c" },
            { "ДÐĎĐΔ", "D" },
            { "дðďđδ", "d" },
            { "ÈÉÊËĒĔĖĘĚΕΈẼẺẸỀẾỄỂỆЕЭ", "E" },
            { "èéêëēĕėęěέεẽẻẹềếễểệеэ", "e" },
            { "Ф", "F" },
            { "ф", "f" },
            { "ĜĞĠĢΓГҐ", "G" },
            { "ĝğġģγгґ", "g" },
            { "ĤĦ", "H" },
            { "ĥħ", "h" },
            { "ÌÍÎÏĨĪĬǏĮİΗΉΊΙΪỈỊИЫ", "I" },
            { "ìíîïĩīĭǐįıηήίιϊỉịиыї", "i" },
            { "Ĵ", "J" },
            { "ĵј", "j" },
            { "ĶΚК", "K" },
            { "ķκк", "k" },
            { "ĹĻĽĿŁΛЛ", "L" },
            { "ĺļľŀłλл", "l" },
            { "МΜ", "M" },
            { "м", "m" },
            { "ÑŃŅŇΝН", "N" },
            { "ñńņňŉνн", "n" },
            { "ÖÒÓÔÕŌŎǑŐƠØǾΟΌΩΏỎỌỒỐỖỔỘỜỚỠỞỢО", "O" },
            { "öòóôõōŏǒőơøǿºοόωώỏọồốỗổộờớỡởợоø", "o" },
            { "ПΠ", "P" },
            { "п", "p" },
            { "ŔŖŘΡР", "R" },
            { "ŕŗřρр", "r" },
            { "ŚŜŞȘŠΣС", "S" },
            { "śŝşșšſσςс", "s" },
            { "ȚŢŤŦТΤΘ", "T" },
            { "țţťŧтτθ", "t" },
            { "ÜÙÚÛŨŪŬŮŰŲƯǓǕǗǙǛŨỦỤỪỨỮỬỰУ", "U" },
            { "üùúûũūŭůűųưǔǖǘǚǜυύϋủụừứữửựу", "u" },
            { "Χ", "X"},
            { "χ", "x" },
            { "ÝŸŶΥΎΫỲỸỶỴЙ", "Y" },
            { "ýÿŷỳỹỷỵй", "y" },
            { "В", "V" },
            { "в", "v" },
            { "Ŵ", "W" },
            { "ŵ", "w" },
            { "ŹŻŽΖЗ", "Z" },
            { "źżžζз", "z" },
            { "ÆǼ", "AE" },
            { "ß", "ss" },
            { "Ĳ", "IJ" },
            { "ĳ", "ij" },
            { "Œ", "OE" },
            { "ƒ", "f" },
            { "ξ", "ks" },
            { "π", "p" },
            { "Β", "V" },
            { "β", "v" },
            { "μ", "m" },
            { "ψ", "ps" },
            { "Ё", "Yo" },
            { "ё", "yo" },
            { "Є", "Ye" },
            { "є", "ye" },
            { "Ї", "Yi" },
            { "Ж", "Zh" },
            { "ж", "zh" },
            { "Х", "Kh" },
            { "х", "kh" },
            { "Ц", "Ts" },
            { "ц", "ts" },
            { "Ч", "Ch" },
            { "ч", "ch" },
            { "Ш", "Sh" },
            { "ш", "sh" },
            { "Щ", "Shch" },
            { "щ", "shch" },
            { "ЪъЬь", "" },
            { "Ю", "Yu" },
            { "ю", "yu" },
            { "Я", "Ya" },
            { "я", "ya" },
            { "’", "`" },
            { "“”", "\"" },
        };

        public static String ConvertToAscii(char c)
        {
            foreach (KeyValuePair<string, string> entry in foreign_characters)
            {
                if (entry.Key.Contains(c))
                {
                    return entry.Value;
                }
            }
            // remove unknown glyphs, e.g. asian scripts (only needed for ~500 entries)
            return "";
        }

        public static string ConvertToAscii(String text)
        {
            ReadOnlySpan<char> normalizedString = text.Normalize(NormalizationForm.FormKC);
            int i = 0;
            Span<byte> span = text.Length < 1000
                ? stackalloc byte[text.Length * 3 + 1]
                : new byte[text.Length * 3 + 1];

            foreach (char c in normalizedString)
            {
                if (CharUnicodeInfo.GetUnicodeCategory(c) != UnicodeCategory.NonSpacingMark)
                {
                    if (Char.IsAscii(c))
                        span[i++] = (byte)c;
                    else
                        foreach (var r in ConvertToAscii(c))
                            span[i++] = (byte)r;
                }
            }

            return Encoding.ASCII.GetString(span.Slice(0, span.IndexOf((byte)0))).Trim();
        }


        static void Main(string[] args)
        {
            var contacts = new List<(int, String, IEnumerable<String>)>();
            var header = "";

            using (var reader = new StreamReader(@"user.csv"))
            {
                header = reader.ReadLine();
                while (!reader.EndOfStream)
                {
                    var line = reader.ReadLine();
                    var values = line.Split(',');

                    int DMRId = int.Parse(values[0]);
                    String callsign = values[1].Trim();
                    var fields = values.Skip(2);

                    contacts.Add((DMRId, callsign, fields));
                }

                Console.WriteLine($"{contacts.Count()} contacts in database.");
            }

            using (var writer = new StreamWriter("output.csv"))
            {
                writer.WriteLine(header);

                foreach ((int DmrID, String call, IEnumerable<String> fields) in contacts)
                {
                    writer.Write($"{DmrID},{call}");

                    foreach (var field in fields)
                        writer.Write($",{ConvertToAscii(field)}");
                    
                    writer.WriteLine("");
                }
            }
        }

    }
}